import { WhitespacePosition, MarkdownItLineNumbersOptions, MarkdownItCollapsedLinesOptions } from '@vuepress/highlighter-helper';
import { LanguageInput, SpecialLanguage, StringLiteralUnion, BundledLanguage, BundledTheme, ThemeRegistrationAny, Highlighter, ShikiTransformer } from 'shiki';
export { bundledLanguages } from 'shiki';
import { Plugin } from 'vuepress/core';

type ShikiLang = LanguageInput | SpecialLanguage | StringLiteralUnion<BundledLanguage>;
type ShikiTheme = StringLiteralUnion<BundledTheme> | ThemeRegistrationAny;
interface ShikiSingleThemeOptions {
    /**
     * The single theme to use
     *
     * @see https://shiki.style/themes
     *
     * @default 'nord'
     */
    theme?: ShikiTheme;
}
interface ShikiDualThemeOptions {
    /**
     * The dark and light themes to use
     *
     * @see https://shiki.style/themes
     */
    themes: {
        dark: ShikiTheme;
        light: ShikiTheme;
    };
}
type ShikiThemeOptions = ShikiDualThemeOptions | ShikiSingleThemeOptions;
type ShikiHighlightOptions = ShikiThemeOptions & {
    /**
     * Languages to be loaded.
     *
     * Shiki does not preload any languages to avoid extra overhead. So you need
     * to specify the languages you want to use.
     *
     * @see https://shiki.style/languages
     */
    langs?: ShikiLang[];
    /**
     * Language alias
     *
     * @see https://shiki.style/guide/load-lang#custom-language-aliases
     */
    langAlias?: Record<string, StringLiteralUnion<BundledLanguage>>;
    /**
     * Fallback language when the specified language is not available.
     *
     * @default 'plain'
     */
    defaultLang?: string;
    /**
     * Function to customize Shiki highlighter instance.
     */
    shikiSetup?: (shiki: Highlighter) => Promise<void> | void;
    /**
     * Shiki transformers
     */
    transformers?: ShikiTransformer[];
    /**
     * Enable highlight lines or not
     *
     * @default true
     */
    highlightLines?: boolean;
    /**
     * Enable notation diff transformer
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformernotationdiff
     */
    notationDiff?: boolean;
    /**
     * Enable notation focus transformer
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformernotationfocus
     */
    notationFocus?: boolean;
    /**
     * Enable notation highlight transformer
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformernotationhighlight
     */
    notationHighlight?: boolean;
    /**
     * Enable notation error level transformer
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformernotationerrorlevel
     */
    notationErrorLevel?: boolean;
    /**
     * Enable notation word highlight transformer
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformernotationwordhighlight
     */
    notationWordHighlight?: boolean;
    /**
     * Enable whitespace
     * - true: enable render whitespace, same of `all`
     * - false: disable render whitespace
     * - 'all': render all whitespace
     * - 'boundary': render leading and trailing whitespace of each line.
     * - 'trailing': render trailing whitespace of each line
     *
     * you are able to use `:whitespace` or `:no-whitespace` or `:whitespace=position` to set single code block
     *
     * position: 'all' | 'boundary' | 'trailing'
     *
     * @default false
     *
     * @see https://shiki.style/packages/transformers#transformerrenderwhitespace
     */
    whitespace?: WhitespacePosition | boolean;
    /**
     * Log level Highlighter language detecter
     *
     * @description defaults to `'debug'` when `--debug` flag is enabled
     *
     * @default 'warn'
     */
    logLevel?: 'debug' | 'silent' | 'warn';
};

interface MarkdownItPreWrapperOptions {
    /**
     * Wrap the `<pre>` tag with an extra `<div>` or not. Do not disable it unless you
     * understand what's it for
     *
     * - Required for line numbers, title display and code block collapsing
     */
    preWrapper?: boolean;
}

/**
 * Options of @vuepress/plugin-shiki
 */
type ShikiPluginOptions = MarkdownItLineNumbersOptions & MarkdownItPreWrapperOptions & Pick<MarkdownItCollapsedLinesOptions, 'collapsedLines'> & ShikiHighlightOptions;

declare const bundledLanguageNames: string[];

declare const shikiPlugin: (_options?: ShikiPluginOptions) => Plugin;

export { type ShikiDualThemeOptions, type ShikiHighlightOptions, type ShikiLang, type ShikiPluginOptions, type ShikiSingleThemeOptions, type ShikiTheme, type ShikiThemeOptions, bundledLanguageNames, shikiPlugin };
